@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2006, International Business Machines Corporation and
+ * Copyright (c) 1997-2010, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -16,7 +16,7 @@
 #include "caltztst.h"
 
 /** 
- * Performs many different tests for DataeFormat and SimpleDateFormat
+ * Performs many different tests for DateFormat and SimpleDateFormat
  **/
 class DateFormatTest: public CalendarTimeZoneTest {
     // IntlTest override
@@ -109,10 +109,6 @@ public:
      */
     virtual void TestBadInput135(void);
  
-private:
-    static const char* parseFormats[];
-    static const char* inputStrings[];
- 
 public:
     /**
      * Verify the correct behavior when parsing an array of inputs against an
@@ -140,7 +136,9 @@ public:
     virtual void TestDateFormatZone146(void);
 
     void TestTimeZoneStringsAPI(void);
- 
+
+    void TestGMTParsing(void);
+
 public: // package
     /**
      * Test the formatting of dates in different locales.
@@ -163,6 +161,12 @@ public: // package
 
     void TestGenericTimeZoneOrder(void);
 
+    void Test6338(void);
+
+    void Test6726(void);
+
+    void Test6880(void);
+
 public:
     /**
      * Test host-specific formatting.
@@ -182,6 +186,30 @@ public:
     void TestStandAloneMonths(void);
 
     void TestQuarters(void);
+    
+    void TestZTimeZoneParsing(void);
+
+    void TestRelativeClone(void);
+    
+    void TestHostClone(void);
+
+    void TestTimeZoneDisplayName(void);
+
+    void TestRoundtripWithCalendar(void);
+
+public:
+    /***
+     * Test Relative Dates
+     */
+     void TestRelative(void);
+/*   void TestRelativeError(void);
+     void TestRelativeOther(void);
+*/
+
+ private:
+      void TestRelative(int daysdelta, 
+                                  const Locale& loc,
+                                  const char *expectChars);
 
  private:
     void expectParse(const char** data, int32_t data_length,
