@@ -1,15 +1,21 @@
 /********************************************************************
- * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * COPYRIGHT:
+ * Copyright (c) 1997-2009, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/coll.h"
 #include "unicode/tblcoll.h"
 #include "unicode/unistr.h"
 #include "unicode/sortkey.h"
 #include "regcoll.h"
 #include "sfwdchit.h"
+#include "testutil.h"
+#include "cmemory.h"
 
 #define ARRAY_LENGTH(array) ((int32_t)(sizeof array / sizeof array[0]))
 
@@ -17,7 +23,13 @@ CollationRegressionTest::CollationRegressionTest()
 {
     UErrorCode status = U_ZERO_ERROR;
 
-    en_us = (RuleBasedCollator *)Collator::createInstance(Locale::US, status);
+    en_us = (RuleBasedCollator *)Collator::createInstance(Locale::getUS(), status);
+    if(U_FAILURE(status)) {
+      delete en_us;
+      en_us = 0;
+      errcheckln(status, "Collator creation failed with %s", u_errorName(status));
+      return;
+    }
 }
 
 CollationRegressionTest::~CollationRegressionTest()
@@ -137,7 +149,8 @@ void CollationRegressionTest::Test4054238(/* char* par */)
 
     // NOTE: The Java code uses en_us to create the CollationElementIterators
     // but I'm pretty sure that's wrong, so I've changed this to use c.
-    c->setDecomposition(Normalizer::DECOMP);
+    UErrorCode status = U_ZERO_ERROR;
+    c->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
     CollationElementIterator *i1 = c->createCollationElementIterator(test3);
     delete i1;
     delete c;
@@ -158,7 +171,7 @@ void CollationRegressionTest::Test4054734(/* char* par */)
             "A\u0001",  ">",    "~\u0002",      // Ensure A and ~ are not compared bitwise
             "\u00C0",   "=",    "A\u0300"       // Decomp should make these equal
         };
- 
+
         String[] nodecomp = {
             "\u00C0",   ">",    "A\u0300"       // A-grave vs. A combining-grave
         };
@@ -172,12 +185,13 @@ void CollationRegressionTest::Test4054734(/* char* par */)
         {0x00c0, 0},      {0x3d, 0}, {0x41, 0x0300, 0}
     };
 
-    
+
+    UErrorCode status = U_ZERO_ERROR;
     RuleBasedCollator *c = (RuleBasedCollator *) en_us->clone();
 
     c->setStrength(Collator::IDENTICAL);
 
-    c->setDecomposition(Normalizer::DECOMP);
+    c->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
     compareArray(*c, decomp, ARRAY_LENGTH(decomp));
 
     delete c;
@@ -189,10 +203,11 @@ void CollationRegressionTest::Test4054734(/* char* par */)
 //
 void CollationRegressionTest::Test4054736(/* char* par */)
 {
+    UErrorCode status = U_ZERO_ERROR;
     RuleBasedCollator *c = (RuleBasedCollator *) en_us->clone();
 
     c->setStrength(Collator::SECONDARY);
-    c->setDecomposition(Normalizer::DECOMP_COMPAT);
+    c->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
 
     static const UChar tests[][CollationRegressionTest::MAX_TOKEN_LEN] =
     {
@@ -206,16 +221,16 @@ void CollationRegressionTest::Test4054736(/* char* par */)
 
 // @bug 4058613
 //
-// Collator::createInstance() causes an ArrayIndexOutofBoundsException for Korean  
+// Collator::createInstance() causes an ArrayIndexOutofBoundsException for Korean
 //
 void CollationRegressionTest::Test4058613(/* char* par */)
 {
     // Creating a default collator doesn't work when Korean is the default
     // locale
-    
+
     Locale oldDefault = Locale::getDefault();
     UErrorCode status = U_ZERO_ERROR;
-    
+
     Locale::setDefault(Locale::getKorean(), status);
 
     if (U_FAILURE(status))
@@ -225,7 +240,7 @@ void CollationRegressionTest::Test4058613(/* char* par */)
     }
 
     Collator *c = NULL;
-    
+
     c = Collator::createInstance("en_US", status);
 
     if (c == NULL || U_FAILURE(status))
@@ -235,10 +250,10 @@ void CollationRegressionTest::Test4058613(/* char* par */)
         delete c;
         return;
     }
-    
+
     // Since the fix to this bug was to turn off decomposition for Korean collators,
     // ensure that's what we got
-    if (c->getDecomposition() != Normalizer::NO_OP)
+    if (c->getAttribute(UCOL_NORMALIZATION_MODE, status) != UCOL_OFF)
     {
       errln("Decomposition is not set to NO_DECOMPOSITION for Korean collator");
     }
@@ -259,7 +274,7 @@ void CollationRegressionTest::Test4059820(/* char* par */)
 
     RuleBasedCollator *c = NULL;
     UnicodeString rules = "< a < b , c/a < d < z";
-    
+
     c = new RuleBasedCollator(rules, status);
 
     if (c == NULL || U_FAILURE(status))
@@ -294,7 +309,7 @@ void CollationRegressionTest::Test4060154(/* char* par */)
     rules += ", i, I";
 
     RuleBasedCollator *c = NULL;
-    
+
     c = new RuleBasedCollator(rules, status);
 
     if (c == NULL || U_FAILURE(status))
@@ -304,7 +319,7 @@ void CollationRegressionTest::Test4060154(/* char* par */)
         return;
     }
 
-    c->setDecomposition(Normalizer::DECOMP);
+    c->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
 
  /*
     String[] tertiary = {
@@ -346,7 +361,7 @@ void CollationRegressionTest::Test4060154(/* char* par */)
     compareArray(*c, secondary, ARRAY_LENGTH(secondary));
 
     delete c;
-};
+}
 
 // @bug 4062418
 //
@@ -357,7 +372,7 @@ void CollationRegressionTest::Test4062418(/* char* par */)
     UErrorCode status = U_ZERO_ERROR;
 
     RuleBasedCollator *c = NULL;
-    
+
     c = (RuleBasedCollator *) Collator::createInstance(Locale::getFrance(), status);
 
     if (c == NULL || U_FAILURE(status))
@@ -408,16 +423,17 @@ void CollationRegressionTest::Test4066189(/* char* par */)
     static const UChar chars2[] = {0x61, 0x0306, 0x0300, 0};
     const UnicodeString test1(chars1);
     const UnicodeString test2(chars2);
+    UErrorCode status = U_ZERO_ERROR;
 
     // NOTE: The java code used en_us to create the
     // CollationElementIterator's. I'm pretty sure that
     // was wrong, so I've change the code to use c1 and c2
     RuleBasedCollator *c1 = (RuleBasedCollator *) en_us->clone();
-    c1->setDecomposition(Normalizer::DECOMP_COMPAT);
+    c1->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
     CollationElementIterator *i1 = c1->createCollationElementIterator(test1);
 
     RuleBasedCollator *c2 = (RuleBasedCollator *) en_us->clone();
-    c2->setDecomposition(Normalizer::NO_OP);
+    c2->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_OFF, status);
     CollationElementIterator *i2 = c2->createCollationElementIterator(test2);
 
     assertEqual(*i1, *i2);
@@ -436,7 +452,7 @@ void CollationRegressionTest::Test4066696(/* char* par */)
 {
     UErrorCode status = U_ZERO_ERROR;
     RuleBasedCollator *c = NULL;
-    
+
     c = (RuleBasedCollator *)Collator::createInstance(Locale::getFrance(), status);
 
     if (c == NULL || U_FAILURE(status))
@@ -559,13 +575,14 @@ void CollationRegressionTest::Test4081866(/* char* par */)
     static const UChar s1[] = {0x41, 0x0300, 0x0316, 0x0327, 0x0315, 0};
     static const UChar s2[] = {0x41, 0x0327, 0x0316, 0x0315, 0x0300, 0};
 
+    UErrorCode status = U_ZERO_ERROR;
     RuleBasedCollator *c = (RuleBasedCollator *) en_us->clone();
     c->setStrength(Collator::TERTIARY);
-    
+
     // Now that the default collators are set to NO_DECOMPOSITION
     // (as a result of fixing bug 4114077), we must set it explicitly
     // when we're testing reordering behavior.  -- lwerner, 5/5/98
-    c->setDecomposition(Normalizer::DECOMP);
+    c->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
 
     if (c->compare(s1,s2) != 0)
     {
@@ -584,7 +601,7 @@ void CollationRegressionTest::Test4087241(/* char* par */)
     UErrorCode status = U_ZERO_ERROR;
     Locale da_DK("da", "DK");
     RuleBasedCollator *c = NULL;
-    
+
     c = (RuleBasedCollator *) Collator::createInstance(da_DK, status);
 
     if (c == NULL || U_FAILURE(status))
@@ -637,7 +654,7 @@ void CollationRegressionTest::Test4092260(/* char* par */)
     UErrorCode status = U_ZERO_ERROR;
     Locale el("el", "");
     Collator *c = NULL;
-    
+
     c = Collator::createInstance(el, status);
 
     if (c == NULL || U_FAILURE(status))
@@ -696,7 +713,7 @@ void CollationRegressionTest::Test4101940(/* char* par */)
     RuleBasedCollator *c = NULL;
     UnicodeString rules = "< a < b";
     UnicodeString nothing = "";
-    
+
     c = new RuleBasedCollator(rules, status);
 
     if (c == NULL || U_FAILURE(status))
@@ -744,6 +761,7 @@ void CollationRegressionTest::Test4103436(/* char* par */)
 //
 void CollationRegressionTest::Test4114076(/* char* par */)
 {
+    UErrorCode status = U_ZERO_ERROR;
     RuleBasedCollator *c = (RuleBasedCollator *) en_us->clone();
     c->setStrength(Collator::TERTIARY);
 
@@ -757,12 +775,12 @@ void CollationRegressionTest::Test4114076(/* char* par */)
         {0xd4db, 0}, {0x3d, 0}, {0x1111, 0x1171, 0x11b6, 0}
     };
 
-    c->setDecomposition(Normalizer::DECOMP);
+    c->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
     compareArray(*c, test1, ARRAY_LENGTH(test1));
 
     // From UTR #15:
     // *In earlier versions of Unicode, jamo characters like ksf
-    //  had compatibility mappings to kf + sf. These mappings were 
+    //  had compatibility mappings to kf + sf. These mappings were
     //  removed in Unicode 2.1.9 to ensure that Hangul syllables are maintained.)
     // That is, the following test is obsolete as of 2.1.9
 
@@ -789,18 +807,19 @@ void CollationRegressionTest::Test4124632(/* char* par */)
 {
     UErrorCode status = U_ZERO_ERROR;
     Collator *coll = NULL;
-    
-    coll = Collator::createInstance(Locale::JAPAN, status);
-    
+
+    coll = Collator::createInstance(Locale::getJapan(), status);
+
     if (coll == NULL || U_FAILURE(status))
     {
         errln("Failed to create collator for Locale::JAPAN");
         delete coll;
+        return;
     }
 
     static const UChar test[] = {0x41, 0x0308, 0x62, 0x63, 0};
     CollationKey key;
-    
+
     coll->getCollationKey(test, key, status);
 
     if (key.isBogus() || U_FAILURE(status))
@@ -820,14 +839,15 @@ void CollationRegressionTest::Test4132736(/* char* par */)
     UErrorCode status = U_ZERO_ERROR;
 
     Collator *c = NULL;
-    
+
     c = Collator::createInstance(Locale::getFrance(), status);
     c->setStrength(Collator::TERTIARY);
-    
+
     if (c == NULL || U_FAILURE(status))
     {
         errln("Failed to create a collator for Locale::getFrance()");
         delete c;
+        return;
     }
 
     static const UChar test1[][CollationRegressionTest::MAX_TOKEN_LEN] =
@@ -859,16 +879,17 @@ void CollationRegressionTest::Test4133509(/* char* par */)
 
 // @bug 4114077
 //
-// Collation with decomposition off doesn't work for Europe 
+// Collation with decomposition off doesn't work for Europe
 //
 void CollationRegressionTest::Test4114077(/* char* par */)
 {
     // Ensure that we get the same results with decomposition off
     // as we do with it on....
-    
+
+    UErrorCode status = U_ZERO_ERROR;
     RuleBasedCollator *c = (RuleBasedCollator *) en_us->clone();
     c->setStrength(Collator::TERTIARY);
-    
+
     static const UChar test1[][CollationRegressionTest::MAX_TOKEN_LEN] =
     {
         {0x00C0, 0},                     {0x3d, 0}, {0x41, 0x0300, 0},            // Should be equivalent
@@ -879,7 +900,7 @@ void CollationRegressionTest::Test4114077(/* char* par */)
         {0x41, 0x0300, 0x0316, 0},         {0x3c, 0}, {0x41, 0x0316, 0x0300, 0}        // No reordering --> unequal
     };
 
-    c->setDecomposition(Normalizer::NO_OP);
+    c->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_OFF, status);
     compareArray(*c, test1, ARRAY_LENGTH(test1));
 
     static const UChar test2[][CollationRegressionTest::MAX_TOKEN_LEN] =
@@ -887,7 +908,7 @@ void CollationRegressionTest::Test4114077(/* char* par */)
         {0x41, 0x0300, 0x0316, 0}, {0x3d, 0}, {0x41, 0x0316, 0x0300, 0}      // Reordering --> equal
     };
 
-    c->setDecomposition(Normalizer::DECOMP);
+    c->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
     compareArray(*c, test2, ARRAY_LENGTH(test2));
 
     delete c;
@@ -895,7 +916,7 @@ void CollationRegressionTest::Test4114077(/* char* par */)
 
 // @bug 4141640
 //
-// Support for Swedish gone in 1.1.6 (Can't create Swedish collator) 
+// Support for Swedish gone in 1.1.6 (Can't create Swedish collator)
 //
 void CollationRegressionTest::Test4141640(/* char* par */)
 {
@@ -907,7 +928,7 @@ void CollationRegressionTest::Test4141640(/* char* par */)
     UErrorCode status = U_ZERO_ERROR;
     int32_t i, localeCount;
     const Locale *locales = Locale::getAvailableLocales(localeCount);
-    
+
     for (i = 0; i < localeCount; i += 1)
     {
         Collator *c = NULL;
@@ -931,7 +952,7 @@ void CollationRegressionTest::Test4141640(/* char* par */)
 
 // @bug 4139572
 //
-// getCollationKey throws exception for spanish text 
+// getCollationKey throws exception for spanish text
 // Cannot reproduce this bug on 1.2, however it DOES fail on 1.1.6
 //
 void CollationRegressionTest::Test4139572(/* char* par */)
@@ -944,7 +965,7 @@ void CollationRegressionTest::Test4139572(/* char* par */)
     UErrorCode status = U_ZERO_ERROR;
     Locale l("es", "es");
     Collator *col = NULL;
-    
+
     col = Collator::createInstance(l, status);
 
     if (col == NULL || U_FAILURE(status))
@@ -968,7 +989,7 @@ void CollationRegressionTest::Test4139572(/* char* par */)
 }
 /* HSYS : RuleBasedCollator::compare() performance enhancements
           compare() does not create CollationElementIterator() anymore.*/
-          
+
 class My4146160Collator : public RuleBasedCollator
 {
 public:
@@ -978,7 +999,7 @@ public:
     CollationElementIterator *createCollationElementIterator(const UnicodeString &text) const;
 
     CollationElementIterator *createCollationElementIterator(const CharacterIterator &text) const;
-    
+
     static int32_t count;
 };
 
@@ -1021,7 +1042,7 @@ void CollationRegressionTest::Test4146160(/* char* par */)
 
     My4146160Collator::count = 0;
     My4146160Collator *mc = NULL;
-    
+
     mc = new My4146160Collator(*en_us, status);
 
     if (mc == NULL || U_FAILURE(status))
@@ -1056,6 +1077,86 @@ void CollationRegressionTest::Test4146160(/* char* par */)
     delete mc;
 #endif
 }
+
+// Ticket 7189
+//
+// nextSortKeyPart incorrect for EO_S1 collation
+static int32_t calcKeyIncremental(UCollator *coll, const UChar* text, int32_t len, uint8_t *keyBuf, int32_t keyBufLen, UErrorCode& status) {
+    UCharIterator uiter;
+    uint32_t state[2] = { 0, 0 };
+    int32_t keyLen;
+    int32_t count = 8;
+
+    uiter_setString(&uiter, text, len);
+    keyLen = 0;
+    while (TRUE) {
+        int32_t keyPartLen = ucol_nextSortKeyPart(coll, &uiter, state, &keyBuf[keyLen], count, &status);
+        if (U_FAILURE(status)) {
+            return -1;
+        }
+        if (keyPartLen == 0) {
+            break;
+        }
+        keyLen += keyPartLen;
+    }
+    return keyLen;
+}
+
+void CollationRegressionTest::TestT7189() {
+    UErrorCode status = U_ZERO_ERROR;
+    UCollator *coll;
+    uint32_t i;
+
+    static const UChar text1[][CollationRegressionTest::MAX_TOKEN_LEN] = {
+    // "Achter De Hoven"
+        { 0x41, 0x63, 0x68, 0x74, 0x65, 0x72, 0x20, 0x44, 0x65, 0x20, 0x48, 0x6F, 0x76, 0x65, 0x6E, 0x00 },
+        // "ABC"
+        { 0x41, 0x42, 0x43, 0x00 },
+        // "HELLO world!"
+        { 0x48, 0x45, 0x4C, 0x4C, 0x4F, 0x20, 0x77, 0x6F, 0x72, 0x6C, 0x64, 0x21, 0x00 }
+    };
+
+    static const UChar text2[][CollationRegressionTest::MAX_TOKEN_LEN] = {
+    // "Achter de Hoven"
+        { 0x41, 0x63, 0x68, 0x74, 0x65, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6F, 0x76, 0x65, 0x6E, 0x00 },
+        // "abc"
+        { 0x61, 0x62, 0x63, 0x00 },
+        // "hello world!"
+        { 0x68, 0x65, 0x6C, 0x6C, 0x6F, 0x20, 0x77, 0x6F, 0x72, 0x6C, 0x64, 0x21, 0x00 }
+    };
+
+    // Open the collator
+    coll = ucol_openFromShortString("EO_S1", FALSE, NULL, &status);
+    if (U_FAILURE(status)) {
+        errln("Failed to create a collator for short string EO_S1");
+        return;
+    }
+
+    for (i = 0; i < sizeof(text1) / (CollationRegressionTest::MAX_TOKEN_LEN * sizeof(UChar)); i++) {
+        uint8_t key1[100], key2[100];
+        int32_t len1, len2;
+
+        len1 = calcKeyIncremental(coll, text1[i], -1, key1, sizeof(key1), status);
+        if (U_FAILURE(status)) {
+            errln(UnicodeString("Failed to get a partial collation key for ") + text1[i]);
+            break;
+        }
+        len2 = calcKeyIncremental(coll, text2[i], -1, key2, sizeof(key2), status);
+        if (U_FAILURE(status)) {
+            errln(UnicodeString("Failed to get a partial collation key for ") + text2[i]);
+            break;
+        }
+
+        if (len1 == len2 && uprv_memcmp(key1, key2, len1) == 0) {
+            errln(UnicodeString("Failed: Identical key\n") + "    text1: " + text1[i] + "\n" + "    text2: " + text2[i] + "\n" + "    key  : " + TestUtility::hex(key1, len1));
+        } else {
+            logln(UnicodeString("Keys produced -\n") + "    text1: " + text1[i] + "\n" + "    key1 : " + TestUtility::hex(key1, len1) + "\n" + "    text2: " + text2[i] + "\n" + "    key2 : "
+                    + TestUtility::hex(key2, len2));
+        }
+    }
+    ucol_close(coll);
+}
+
 void CollationRegressionTest::compareArray(Collator &c,
                                            const UChar tests[][CollationRegressionTest::MAX_TOKEN_LEN],
                                            int32_t testCount)
@@ -1129,7 +1230,7 @@ void CollationRegressionTest::assertEqual(CollationElementIterator &i1, Collatio
         if (c1 != c2)
         {
             UnicodeString msg, msg1("    ");
-            
+
             msg += msg1 + count;
             msg += ": strength(0x";
             appendHex(c1, 8, msg);
@@ -1153,40 +1254,47 @@ void CollationRegressionTest::runIndexedTest(int32_t index, UBool exec, const ch
         logln("Collation Regression Tests: ");
     }
 
-    switch (index)
-    {
-        case  0: name = "Test4048446"; if (exec) Test4048446(/* par */); break;
-        case  1: name = "Test4051866"; if (exec) Test4051866(/* par */); break;
-        case  2: name = "Test4053636"; if (exec) Test4053636(/* par */); break;
-        case  3: name = "Test4054238"; if (exec) Test4054238(/* par */); break;
-        case  4: name = "Test4054734"; if (exec) Test4054734(/* par */); break;
-        case  5: name = "Test4054736"; if (exec) Test4054736(/* par */); break;
-        case  6: name = "Test4058613"; if (exec) Test4058613(/* par */); break;
-        case  7: name = "Test4059820"; if (exec) Test4059820(/* par */); break;
-        case  8: name = "Test4060154"; if (exec) Test4060154(/* par */); break;
-        case  9: name = "Test4062418"; if (exec) Test4062418(/* par */); break;
-        case 10: name = "Test4065540"; if (exec) Test4065540(/* par */); break;
-        case 11: name = "Test4066189"; if (exec) Test4066189(/* par */); break;
-        case 12: name = "Test4066696"; if (exec) Test4066696(/* par */); break;
-        case 13: name = "Test4076676"; if (exec) Test4076676(/* par */); break;
-        case 14: name = "Test4078588"; if (exec) Test4078588(/* par */); break;
-        case 15: name = "Test4079231"; if (exec) Test4079231(/* par */); break;
-        case 16: name = "Test4081866"; if (exec) Test4081866(/* par */); break;
-        case 17: name = "Test4087241"; if (exec) Test4087241(/* par */); break;
-        case 18: name = "Test4087243"; if (exec) Test4087243(/* par */); break;
-        case 19: name = "Test4092260"; if (exec) Test4092260(/* par */); break;
-        case 20: name = "Test4095316"; if (exec) Test4095316(/* par */); break;
-        case 21: name = "Test4101940"; if (exec) Test4101940(/* par */); break;
-        case 22: name = "Test4103436"; if (exec) Test4103436(/* par */); break;
-        case 23: name = "Test4114076"; if (exec) Test4114076(/* par */); break;
-        case 24: name = "Test4114077"; if (exec) Test4114077(/* par */); break;
-        case 25: name = "Test4124632"; if (exec) Test4124632(/* par */); break;
-        case 26: name = "Test4132736"; if (exec) Test4132736(/* par */); break;
-        case 27: name = "Test4133509"; if (exec) Test4133509(/* par */); break;
-        case 28: name = "Test4139572"; if (exec) Test4139572(/* par */); break;
-        case 29: name = "Test4141640"; if (exec) Test4141640(/* par */); break;
-        case 30: name = "Test4146160"; if (exec) Test4146160(/* par */); break;
-        default: name = ""; break;
+    if(en_us) {
+      switch (index)
+      {
+          case  0: name = "Test4048446"; if (exec) Test4048446(/* par */); break;
+          case  1: name = "Test4051866"; if (exec) Test4051866(/* par */); break;
+          case  2: name = "Test4053636"; if (exec) Test4053636(/* par */); break;
+          case  3: name = "Test4054238"; if (exec) Test4054238(/* par */); break;
+          case  4: name = "Test4054734"; if (exec) Test4054734(/* par */); break;
+          case  5: name = "Test4054736"; if (exec) Test4054736(/* par */); break;
+          case  6: name = "Test4058613"; if (exec) Test4058613(/* par */); break;
+          case  7: name = "Test4059820"; if (exec) Test4059820(/* par */); break;
+          case  8: name = "Test4060154"; if (exec) Test4060154(/* par */); break;
+          case  9: name = "Test4062418"; if (exec) Test4062418(/* par */); break;
+          case 10: name = "Test4065540"; if (exec) Test4065540(/* par */); break;
+          case 11: name = "Test4066189"; if (exec) Test4066189(/* par */); break;
+          case 12: name = "Test4066696"; if (exec) Test4066696(/* par */); break;
+          case 13: name = "Test4076676"; if (exec) Test4076676(/* par */); break;
+          case 14: name = "Test4078588"; if (exec) Test4078588(/* par */); break;
+          case 15: name = "Test4079231"; if (exec) Test4079231(/* par */); break;
+          case 16: name = "Test4081866"; if (exec) Test4081866(/* par */); break;
+          case 17: name = "Test4087241"; if (exec) Test4087241(/* par */); break;
+          case 18: name = "Test4087243"; if (exec) Test4087243(/* par */); break;
+          case 19: name = "Test4092260"; if (exec) Test4092260(/* par */); break;
+          case 20: name = "Test4095316"; if (exec) Test4095316(/* par */); break;
+          case 21: name = "Test4101940"; if (exec) Test4101940(/* par */); break;
+          case 22: name = "Test4103436"; if (exec) Test4103436(/* par */); break;
+          case 23: name = "Test4114076"; if (exec) Test4114076(/* par */); break;
+          case 24: name = "Test4114077"; if (exec) Test4114077(/* par */); break;
+          case 25: name = "Test4124632"; if (exec) Test4124632(/* par */); break;
+          case 26: name = "Test4132736"; if (exec) Test4132736(/* par */); break;
+          case 27: name = "Test4133509"; if (exec) Test4133509(/* par */); break;
+          case 28: name = "Test4139572"; if (exec) Test4139572(/* par */); break;
+          case 29: name = "Test4141640"; if (exec) Test4141640(/* par */); break;
+          case 30: name = "Test4146160"; if (exec) Test4146160(/* par */); break;
+		  case 31: name = "TestT7189";   if (exec) TestT7189(); break;
+          default: name = ""; break;
+      }
+    } else {
+      dataerrln("Class collator not instantiated");
+      name = "";
     }
 }
 
+#endif /* #if !UCONFIG_NO_COLLATION */
