@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2010, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -11,6 +11,10 @@
 #ifndef _TSTNORM
 #define _TSTNORM
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_NORMALIZATION
+
 #include "unicode/normlzr.h"
 #include "intltest.h"
 
@@ -35,12 +39,27 @@ public:
     void TestPreviousNext(void);
     void TestNormalizerAPI(void);
     void TestConcatenate(void);
+    void TestCompare(void);
+    void FindFoldFCDExceptions();
+    void TestSkippable();
+    void TestCustomComp();
+    void TestCustomFCC();
 
 private:
     UnicodeString canonTests[24][3];
     UnicodeString compatTests[11][3];
     UnicodeString hangulCanon[2][3];
 
+    void
+    TestPreviousNext(const UChar *src, int32_t srcLength,
+                     const UChar32 *expext, int32_t expectLength,
+                     const int32_t *expectIndex, // its length=expectLength+1
+                     int32_t srcMiddle, int32_t expectMiddle,
+                     const char *moves,
+                     UNormalizationMode mode,
+                     const char *name);
+
+    int32_t countFoldFCDExceptions(uint32_t foldingOptions);
 
     //------------------------------------------------------------------------
     // Internal utilities
@@ -62,4 +81,6 @@ private:
 
 };
 
+#endif /* #if !UCONFIG_NO_NORMALIZATION */
+
 #endif // _TSTNORM
